import com.urbancode.air.*;
import com.urbancode.air.plugin.automation.*;
import org.codehaus.jettison.json.JSONArray;
import org.codehaus.jettison.json.JSONObject;
import javax.ws.rs.core.UriBuilder;

//------------------------------------------------------------------------------
// GET ALL INPUT PARAMETERS
//------------------------------------------------------------------------------
final def workDir = new File('.').canonicalFile
final def javaHome = System.getenv('JAVA_HOME')
final def webUrl = System.getenv('WEB_URL')
final def authToken = System.getenv('AUTH_TOKEN')
final def buildLifeId = System.getenv('BUILD_LIFE_ID')
final def pluginHome = System.getenv("PLUGIN_HOME");

final def apTool = new AirPluginTool(this.args[0], this.args[1]);
final def stepProps = apTool.getStepProperties();
final def CommandHelper cmdHelper = new CommandHelper(workDir);

final def javaFile = new File(new File(javaHome), 'bin/java')
final def udClientJarFile = new File(new File(pluginHome), 'lib/udeploy-client.jar')

def url = stepProps['automation/url'];
def user = stepProps['automation/user'];
def password = stepProps['automation/passScript'];
def componentName = stepProps['componentName'];
def versionName = stepProps['versionName'];
def propertiesTextArea = stepProps['propertiesTextArea'];
if (password == null || password.trim() == "") {
    password = stepProps['automation/password'];
}

def propertiesToSet = new Properties()
propertiesToSet.load(new StringReader(propertiesTextArea))

if (propertiesToSet.isEmpty()) {
    println "No properties defined to set"
    return
}

def httpHelper = new HttpHelper();
if (user && password) {
    httpHelper.setUsernameAndPassword(user, password)
}

// get the component version
def versionsUrl = UriBuilder.fromUri(url).segment("rest", "deploy", "component", componentName, "versions", "false").build().toString()
def versionsJson = new JSONArray(httpHelper.getContent(versionsUrl))

def versionId
for (int versionIndex = 0; versionIndex < versionsJson.length(); versionIndex++) {
    def versionJson = versionsJson.getJSONObject(versionIndex)
    if (versionName.equals(versionJson.getString("name"))) {
        versionId = versionJson.getString("id")
        break
    }
}

if (!versionId) {
    throw new Exception("Version ${versionName} not found on component ${componentName}")
}

// you can only create properties on the version that are defined on the component
def componentUrl = UriBuilder.fromUri(url).segment("rest", "deploy", "component", componentName).build().toString()
//println "Component URL: ${componentUrl}"
def componentContent = httpHelper.getContent(componentUrl)
//println "Component JSON:\n${componentContent}"

def componentJson = new JSONObject(componentContent)
def componentId = componentJson.getString("id")
def propSheetDefPath = componentJson.getJSONObject("versionPropSheetDef").getString("path")
propSheetDefPath += "." + componentJson.getJSONObject("versionPropSheetDef").getString("version")
def propDefsUrl = UriBuilder.fromUri(url).segment("property", "propSheetDef", propSheetDefPath, "propDefs").build().toString()
//println "Component Version Prop Defs URL: ${propDefsUrl}"

def propSheetDefContent = httpHelper.getContent(propDefsUrl)
//println "Component Version Prop Sheet Def JSON:\n${propSheetDefContent}"
def propSheetDefJson = new JSONArray(propSheetDefContent)

def propertiesToCreate = []
for (def propertyName : propertiesToSet.keySet()) {
    println "Looking for property definition ${propertyName}"
    def foundProperty = false
    for (int psIndex = 0; psIndex < propSheetDefJson.length(); psIndex++) {
        def propSheetDef = propSheetDefJson.getJSONObject(psIndex)
        if (propertyName.equals(propSheetDef.getString("name"))) {
            foundProperty = true
            break
        }
    }
    if (!foundProperty) {
        propertiesToCreate.add(propertyName)
    }
}

if (!propertiesToCreate.empty) {
    println "Need to create property definitions: ${propertiesToCreate}"
    for (def propertyToCreate : propertiesToCreate) {
        // {"id":"caa95ea5-cf17-4369-80a4-9abcbffe0da0","name":"my_prop_def","label":"my_prop_def","type":"TEXT",
        // "value":"","required":false,"description":"","inherited":false}
        JSONObject propertyCreateJson = new JSONObject()
        propertyCreateJson.put("name", propertyToCreate)
        propertyCreateJson.put("label", "")
        propertyCreateJson.put("type", "TEXT")
        propertyCreateJson.put("value", "")
        propertyCreateJson.put("required", false)
        propertyCreateJson.put("description", "")
        propertyCreateJson.put("inherited", false)
        httpHelper.putContent(propDefsUrl, propertyCreateJson.toString())
        println "Created property definition for ${propertyToCreate}"
    }
}
    

// now set the properties
def versionUrl = UriBuilder.fromUri(url).segment("rest", "deploy", "version", versionId).build().toString()
def versionJson = new JSONObject(httpHelper.getContent(versionUrl))
//println "Component Version:\n${versionJson}"

def propSheetPathToUpdate
def versionPropSheetsJson = versionJson.getJSONArray("propSheets")
for (int vpsIndex = 0; vpsIndex < versionPropSheetsJson.length(); vpsIndex++) {
    def versionPropSheetJson = versionPropSheetsJson.getJSONObject(vpsIndex)
    if (!versionPropSheetJson.has("name")) {
        propSheetPathToUpdate = versionPropSheetJson.getString("path") + "." + versionPropSheetJson.getString("version")
        break
    }
}

if (!propSheetPathToUpdate) {
    throw new Exception("Did not find the location to update properties for the component version.")
}

def versionPropSheetUrl = UriBuilder.fromUri(url).segment("property", "propSheet", propSheetPathToUpdate).build().toString()
def versionPropSheetJson = new JSONObject(httpHelper.getContent(versionPropSheetUrl))
def propertiesJson = versionPropSheetJson.getJSONArray("properties")
//println "Component Version Properties:\n${versionPropSheetJson}"

for (def propertyName : propertiesToSet.keySet()) {
    def propertyValue = propertiesToSet.getProperty(propertyName) ?: ""
    boolean propertyExists = false
    for (int propIndex = 0; propIndex < propertiesJson.length(); propIndex++) {
        def propertyJson = propertiesJson.getJSONObject(propIndex)
        if (propertyName.equals(propertyJson.getString("name"))) {
            propertyExists = true
            propertyJson.put("value", propertyValue)
            
            println "Updating property ${propertyName} with value ${propertyValue}"
            def updatePropUrl = UriBuilder.fromUri(url).segment("property", "propSheet", propSheetPathToUpdate,
                "propValues", propertyName).build().toString()
            httpHelper.putContent(updatePropUrl, propertyJson.toString())
            break
        }
    }
    if (!propertyExists) {
        // create it
        def propertyJson = new JSONObject()
        propertyJson.put("name", propertyName)
        propertyJson.put("value", propertyValue)
        propertyJson.put("description", "")
        propertyJson.put("secure", false)
            
        println "Creating property ${propertyName} with value ${propertyValue}"
        def createPropUrl = UriBuilder.fromUri(url).segment("property", "propSheet", propSheetPathToUpdate,
            "propValues").build().toString()
        httpHelper.putContent(createPropUrl, propertyJson.toString())
    }
}
println "${componentName} ${versionName} properties updated successfully"